#include<omp.h>
#include<math.h>
#include<stdio.h>
#include <gsl/gsl_rng.h>
#include <gsl/gsl_randist.h>
#include"nrutil.h" 
#include"nr.h"
#include"declare.h"

void get_alm(double **Cl_all_ell)
{
    /****************************************************************************************/
    /* get the alm randomly, out of the set of Cl, out to l=L_MAX_CALIB                    **/
    /* as usual m=0 is ran_gauss(sqrt(Cl)), while m>0 RE and IM are ran_gauss(sqrt(0.5*Cl)) */
    /****************************************************************************************/

    int l, m, iz,  alpha;
    double Cl;

    /***************************************************************************************/
    /** doesn't really matter local or global rng, since you are only running this once   **/
    /***************************************************************************************/
    gsl_rng * r;
    const gsl_rng_type * T;
    T = gsl_rng_default;
    r = gsl_rng_alloc (T);    

    //gsl_rng_set (r, 1); // to change seed!    
    //gsl_rng_env_setup();     
    //printf ("seed = %lu\n", gsl_rng_default_seed);

    for(iz=1; iz<=Z_BINS; iz++)
    {
        for (l = 1; l <= L_MAX_CALIB; l++)
        {
            alpha = ij_to_alpha(iz, iz);
            Cl = Cl_all_ell[alpha][l];
            
            // m=0
            alm_RE[iz][l+1][1] = gsl_ran_gaussian(r, sqrt(Cl));
            
            //printf("L=%d Cl=%f\n", l, Cl);
            //printf("%f\n", alm_RE[iz][l+1][1]/sqrt(Cl));
            
            for (m = 1; m <= l; m++)
            {
                alm_RE[iz][l+1][m+1] = gsl_ran_gaussian(r, sqrt(0.5*Cl));
                alm_IM[iz][l+1][m+1] = gsl_ran_gaussian(r, sqrt(0.5*Cl));
                //printf("%f %f\n", alm_RE[iz][l+1][m+1]/sqrt(0.500*Cl), alm_IM[iz][l+1][m+1] /sqrt(0.500*Cl));
            }
        }
    }
    gsl_rng_free (r) ;
}
void assign_clm_given_fixed_rms(double delta_c, int lmax, double ***clm_array)
{
    /*********************************************************************************/
    // For a given rms = sqrt(variance), put the power EQUALLY into every ell
    // range: 1 <= ell <= lmax
    // conversion from mag to clm is not done here, but in calculate_Tl!
    // (so here we are really assigning clm/(ln(10)*s(z)) in cases of mag-error choice)
    /*********************************************************************************/
    
    int l1, m1;
    double delta_c_this_ell;

    
    for (l1 = 1; l1 <= lmax;  l1 ++)// go over all l1
    {
        // split equally among (l1+1) values of m for that l1
        delta_c_this_ell = delta_c*sqrt(4*M_PI) / sqrt(lmax * (l1+1.0));
        
        for(m1=0; m1<=l1; m1++) // go over all m
        {
            // define this element of contamination, c_l0 = delta c*sqrt(4*PI/(2l+1))
            if (m1 == 0)
                clm_array[l1+1][m1+1][1] =  delta_c_this_ell;
            else // equipartition in RE and IM - doesn't matter much really if you lump it all in RE etc
            {
                clm_array[l1+1][m1+1][1] = delta_c_this_ell/sqrt(2.0);
                clm_array[l1+1][m1+1][2] = delta_c_this_ell/sqrt(2.0);
            }
            //printf("%d %d %f %f\n", l1, m1, clm_array[l1+1][m1+1][1], clm_array[l1+1][m1+1][2]) ;
        }
    }
}
void assign_clm_given_Cl_calib(double *Cl_calib, double ***clm_array)
{
    /****************************************************************************/
    /** given the rms=sqrt(variance) of calibration-field c(n) fluctuations,   **/
    /** draw the constent harmonic coefficients of calibration, clm            **/
    /** constraint: variance is fixed at every multipole l1                    **/
    /****************************************************************************/
    // conversion from mag to clm is not done here, but in calculate_Tl!
    // (so here we are really assigning clm/(ln(10)*s(z)) in cases of mag-error choice)
    /****************************************************************************/

    int l1, m1;
    double delta_c_this_ell;
        
    for (l1 = 1; l1 <= L_MAX_CALIB;  l1++)// go over all l1
    {
        // clm is square root of Cl (of calibration)
        delta_c_this_ell = sqrt(Cl_calib[l1]);
        
        for(m1=0; m1<=l1; m1++) // go over all m
        {
            // define this element of contamination, c_l0 = delta c*sqrt(4*PI/(2l+1))
            if (m1 == 0)
                clm_array[l1+1][m1+1][1] =  delta_c_this_ell;
            else // equipartition in RE and IM [wouldn't matter much if picked random numbers, or you lumped it all in RE...]
            {
                clm_array[l1+1][m1+1][1] = delta_c_this_ell/sqrt(2.0);
                clm_array[l1+1][m1+1][2] = delta_c_this_ell/sqrt(2.0);
            }
            //printf("%d %d %f %f\n", l1, m1, clm_array[l1+1][m1+1][1], clm_array[l1+1][m1+1][2] );
        }
    }

}
int ij_to_alpha(int i, int j)
{
    /*************************************************************************/
    /** convert from redshift-bins i and j to single variable alpha         **/
    /** assumes that i goes from 1 to Z_BINS, and j goes from i to Z_BINS-1 **/
    /*************************************************************************/
    return (Z_BINS*(i-1) - i*(i-1)/2 + j);
}
int file_eof_linecount(char *filename)
{
    int NL=0, c;
  FILE *ifp;
  ifp=fopen(filename, "r");
  if (ifp == NULL)
  {
      fprintf(stderr, "Error: failed to open file %s\n", filename);
      exit(EXIT_FAILURE);
  }

  do      
  {
      c = fgetc(ifp);
      if (c == '\n') NL++;
  }
  while (c != EOF);
  
  //printf("Number of lines = %d\n",  NL);
  fclose(ifp);
  return NL;
}
int count_words_in_row_of_file(char *filename)
{
    /*******************************************************************************/
    // counts number of different 'words' (we care about numbers, int or double)
    // in the given file
    /*******************************************************************************/
    int count, prev,firstpass=1;
    FILE *f;
    char s;
 
    count = 0;
 
    f=fopen(filename,"r");    
    while ((s = fgetc(f)) != '\n')
    {
        if (firstpass == 1)
        {
            firstpass=0;
            if (s == 32) // very first char read is empty
                prev = 0;
            else // encountered symbol right at beginning
            {
                prev = 1;
                count++;
            }
        }

        if ((s != 32) && (prev == 0)) // if symbol here and empty previous spot
        {
            count++;
            //printf("s=%d prev=%d   count=%d\n", s, prev, count);
            prev = 1;
        }
        else if (s == 32)
            prev=0;  // empty here
    }
 
    fclose(f);

    //printf ("\n%d words in 1st row of file %s\n", count, filename);
 
    return count;  
}
